/* General */
body {
  background: #fcfcfc;
}
#author {
  font-size: 0.8em;
}

#header-section {
  background: #fff;
  border-bottom: 1px solid #eee;
  padding: 10px;
}
#header-title {
  margin-bottom: 5px;
  font-weight: bold;
  font-size: 16px;
}
#header-section .form-group,
#header-section .checkbox {
  margin-bottom: 0;
}

#selected-cols-row {
  position: relative;
  -webkit-user-select: none;
  -moz-user-select: none;
  -khtml-user-select: none;
  -ms-user-select: none;
}

#addColBtn,
#removeColBtn,
#selected-cols-row .col {
  cursor: pointer;
  vertical-align: middle;
  border: 1px solid #eee;
  font-weight: bold;
  display: inline-block;
  width: 35px;
  height: 35px;
  line-height: 35px;
  text-align: center;
  margin: 3px 0;
}
#removeColBtn {
  margin-right: 10px;
}
#removeColBtn.disabled {
  color: #bbb !important;
  background: #eee !important;
  cursor: not-allowed;
}
#addColBtn,
#removeColBtn {
  font-size: 22px;
  color: #333;
  background: #FAFAFA;
}
#addColBtn:hover,
#removeColBtn:hover {
  background: #EEE
}
#addColBtn:active,
#removeColBtn:active {
  background: #DDD;
}

#selected-cols-row .col {
  font-size: 15px;
  margin-right: 2px;
  color: black;
}
#selected-cols-row .col.col-dark {
  color: #ddd;
}
#selected-cols-row .col:hover {
  box-shadow: 0 0 0 1px #fafafa, 0 0 0 3px #AAA;
}
#selected-cols-row .col.selected {
  cursor: default;
  box-shadow: 0 0 0 1px #fafafa, 0 0 0 3px #666;
}
.col-transparent-box {
  background-image: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAMAAAC67D+PAAAAElBMVEXs7Ozt7e3u7u79/f3+/v7////P4ouPAAAAI0lEQVQIW2NkAAJGZkYQSZAJIcAkC5D4//8/iMkKYv4ligkAoE4PJIKs1ewAAAAASUVORK5CYII=);
}
.selected-col-inner {
  height: 100%;
  width: 100%;
}

.ksh {
  margin: 10px 0;
  font-size:16px;
}
.ksh-left {
  min-width: 120px;
  display: inline-block;
}
.ksh-key {
  color: #555;
  padding: 2px 8px;
  text-shadow: 0px 1px white;
  display: inline-block;
  border-radius: 3px;
  background: #eee;
  background: linear-gradient(#f5f5f5 0%, #eee 100%);
  box-shadow: 0px 1px 0 #ccc;
  border: 1px solid #ddd;
}


/* Tab 1 - any colour */

#anycolarea {
  text-align: center;
}
#anycolarea .shiny-input-container {
  text-align: left;
  margin: 0 auto;
}

/* R colours in tab 2 & 3 */

.rcol {
  width: 25px;
  height: 25px;
  border: 1px solid #DDD;
  cursor: pointer;
  display: inline-block;
}
.rcol:hover {
  box-shadow: 0 0 0 1px #fafafa, 0 0 0 3px #aaa;
}
.rcolbox {
  margin-right: 10px;
  margin-top: 10px;
  display: inline-block;
  vertical-align: top;
}
.rcolbig {
  height: 80px;
  display: block;
}
.rcolbig:last-child {
  margin-right: 0;
}
.rcolbox,
.rcolbig,
.rcolname {
  width: 80px;
}
.rcolname {
  word-break: break-all;
  font-size: 0.9em;
  font-style: italic;
}
#rclosecolsSection,
#allColsSection {
  padding-bottom: 10px;
}

/* Tab 2 - similar R colours */

#customSliderContainer .irs-grid-text,
#customSliderContainer .irs-grid-pol {
  display: none;
}
#customSliderContainer .irs-single,
#customSliderContainer .irs-bar,
#customSliderContainer .irs-bar-edge {
  background: #666;
  border-color: #333;
}

/* Tab 3 - all R colours */

#allColsSection {
  margin-top: 5px;
}
#allcols-spinner {
  position: absolute;
  top: 100px;
  left: 50%;
  margin-left: -33px;
}
